#include <Engine/Level/Scene.h>

bool Engine::Scene::EntityListContains(const Entity* entity) const {
    for (auto* e : EntityList)
        if (e == entity)
            return true;
    return false;
}

bool Engine::Scene::FixedListContains(const Fixed* fixed) const {
    for (auto* f : FixedList)
        if (f == fixed)
            return true;
    return false;
}

size_t Engine::Scene::FixedCount() const {
    return FixedList.size();
}

size_t Engine::Scene::EntityCount() const {
    return EntityList.size();
}

void Engine::Scene::Update() {
    for (auto& e : EntityList)
        e->Update();
}

void Engine::Scene::Render(RenderTarget* render_target) {
    for (auto& f : FixedList)
        if (f->Enabled())
            f->Render();

    // TODO Render order. In this system it's not possible for child entities to be rendered before the parent.

    J2D::Begin(render_target, true);
    for (auto& e : EntityList)
        if (auto* r = dynamic_cast<Renderable*>(e))
            r->Render();

    if (HeadsUpDisplay)
        HeadsUpDisplay->Render();
    J2D::End();
}

Engine::Scene::~Scene() {
    for (auto* f : FixedList)
        delete f;

    for (auto* e : EntityList)
        delete e;

    delete HeadsUpDisplay;
}

void Engine::Scene::Unload() {
    for (auto* f : FixedList)
        delete f;

    for (auto* e : EntityList)
        delete e;

    delete HeadsUpDisplay;
}

void Engine::Scene::AppendEntity(Entity* entity) {
    if (!EntityListContains(entity))
        EntityList.push_back(entity);
}

void Engine::Scene::AppendFixed(Fixed* fixed) {
    if (!FixedListContains(fixed))
        FixedList.push_back(fixed);
}

void Engine::Scene::DestroyEntity(Entity *entity) {
    auto it = std::find(EntityList.begin(), EntityList.end(), entity);
    if (it != EntityList.end())
        delete *it, EntityList.erase(it);
}

void Engine::Scene::DestroyFixed(Fixed* fixed) {
    auto it = std::find(FixedList.begin(), FixedList.end(), fixed);
    if (it != FixedList.end())
        delete *it, FixedList.erase(it);
}

void Engine::Scene::RemoveEntity(Entity* entity) {
    auto it = std::find(EntityList.begin(), EntityList.end(), entity);
    if (it != EntityList.end())
        EntityList.erase(it);
}


void Engine::Scene::RemoveFixed(Fixed* fixed) {
    auto it = std::find(FixedList.begin(), FixedList.end(), fixed);
    if (it != FixedList.end())
        FixedList.erase(it);
}

std::string Engine::Scene::GetName() const {
    return name;
}

